
// Demo-Programm: Telefonlistenverwaltung unter Verwendung von C++-
//                Stream-Ein-/Ausgabefunktionen

#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>
#include <string.h>

class tellist
{
public:
    static int zahl;                    // Zahl der Eintragungen

    tellist ()                          // Default-Konstruktor
        { *_name = 0; *_nummer = 0; }

    tellist (char *Name, char *Nummer)
    {
        strncpy (_name, Name, 40);
        _name[39] = 0;
        strncpy (_nummer, Nummer, 20);
        _nummer[19] = 0;
    }

    void disp ()
    {
        cout << setiosflags (ios::left) << "Name: " << setw (40) << _name
            << ": Nummer: " << setw (20) << _nummer << endl 
            << resetiosflags (ios::left); 
    }

private:
    char _name[40], _nummer[20];
};

// Funktionen fr die Stream-Ein-/Ausgabe

void lesen (tellist *liste, int max_zahl)
{
    int bytes_rd = 0;                   // Anzahl der gelesenen Bytes

    ifstream is ("TELLIST.DAT", ios::in | ios::binary | ios::nocreate);

    if (is)                             // erfolgreich geffnet
    {
        is.read ((char*) liste, max_zahl * sizeof(tellist));

        bytes_rd = is.gcount ();
    }

    tellist::zahl = bytes_rd / sizeof (tellist);

    cout << tellist::zahl << " Eintragungen gelesen.\n";
}

#if 0   // wenn die GNU-Bibliotheksroutinen keinen fatalen Bug htten...

void schreiben (tellist *liste)
{
    int bytes_wr = 0;                   // geschriebene Bytes

    ofstream os ("TELLIST.DAT", ios::out | ios::trunc | ios::binary);

    if (os)                             // erfolgreich geffnet
    {
        os.write ((char*) liste, tellist::zahl * sizeof(tellist));

        bytes_wr = os.tellp();
    }

    cout << bytes_wr / sizeof (tellist) << " Eintragungen "
        "geschrieben.\n";
}

#endif


// Work-around: Standard-C-Ausgabe-Funktion, weil GNU C++ Stream-Datei-
// Ausgabefunktionen defekt sind

#include <osfcn.h>              // fr ungepufferten I/O
#include <fcntl.h>              // fr ungepufferten I/O
#include <sys/stat.h>           // fr ungepufferten I/O

void schreiben (tellist *liste)
{
    int fd;                             // File Descriptor
    int bytes_wr = 0;                   // geschriebene Bytes

    if ((fd = open ("TELLIST.DAT", O_WRONLY | O_CREAT | O_BINARY, S_IREAD
        | S_IWRITE)) > 0)
            // erfolgreich geffnet
    {
        bytes_wr = write (fd, liste, tellist::zahl * sizeof(tellist));

        close (fd);
    }

    cout << bytes_wr / sizeof (tellist) << " Eintragungen "
        "geschrieben.\n";
}


int tellist::zahl = 0;                  // statisches Element

const int max_zahl = 20;                // maximale Zahl der Eintrge

tellist liste[max_zahl];

int main ()
{
    lesen (liste, max_zahl);            // Liste einlesen

    for (int i = 0; i < tellist::zahl; i++)
        liste[i].disp();                // Liste ausschreiben

    cout << "\nNeue Eintragungen eingeben; Ende mit leerem "
        "Namensfeld:\n";

    while (tellist::zahl < max_zahl)
    {
        char buffer1[128], buffer2[128];

        cout << "Name:    ";
        cin.getline (buffer1, 128);

        if (! *buffer1)                 // leerer String
            break;

        cout << "Nummer:  ";
        cin.getline (buffer2, 128);

        if (! *buffer2)                 // leerer String
            break;

        liste[tellist::zahl++] = tellist (buffer1, buffer2);
    }

    for (i = 0; i < tellist::zahl; i++)
        liste[i].disp();                // Liste ausschreiben

    schreiben (liste);                  // Liste in Datei schreiben
}
