// Demo-Programm: Graphik in GNU Standard-C
//                bersetzen mit "gpp graphtst -lgr"

#include <graphics.h>
#include <pc.h>                         // fr getkey()
#include <keys.h>
#include <stdio.h>

void Palette_16 (void);
void Palette_16_Reinigen (void);
void Testbild_16 (void);
void Palette_256 (void);
void Palette_256_Reinigen (void);
void Testbild_256 (void);

int c[256];                             // Farben

int main ()
{
    GrSetMode (GR_default_graphics);    // wechsle in Graphik-Mode

    Palette_16();

    Testbild_16();

    Palette_16_Reinigen();

    getkey ();                          // warte auf Tastendruck

    GrSetMode (GR_default_graphics);    // Schirm lschen

    Palette_256();

    Testbild_256();

    Palette_256_Reinigen();

    getkey ();                          // warte auf Tastendruck

    GrSetMode (GR_default_text);
}


void Palette_16 (void)
{
    // Ausgabe in 16 Farben

    for (int i = 0; i < 16; i++)
    {
        // GRAllocColor bentigt R/G/B-Werte zwischen 0 und 255

        int b = (i & 1) && 1;   // blau - 0 oder 1
        int g = (i & 2) && 1;   // grn
        int r = (i & 4) && 1;   // rot

        if (i & 8)              // hohe Intensitt
        {
            b *= 0xff;          // 0 oder 255
            g *= 0xff;
            r *= 0xff;
        }

        else                    // normale Intensitt
        {
            b *= 0x7f;          // 0 oder 127
            g *= 0x7f;
            r *= 0x7f;
        }

        if (i == 8)
            b = g = r = 0x80;

        c[i] = GrAllocColor (r, g, b);
    }
}


void Palette_16_Reinigen (void)
{
    for (int i = 0; i < 16; i++)
        GrFreeColor (c[i]);
}


void Testbild_16 (void)
{
    int dx = GrSizeX() / 32;
    int dy = GrSizeY() / 32;

    int mx = GrSizeX() / 2;             // Bildschirm-Mittelpunkt
    int my = GrSizeY() / 2;

    int sx, sy;                         // Startpunkte
    int i;

    for (i = 0, sx = 0, sy = 0 ; i < 16; i++, sx += dx)
        GrLine (mx, my, sx, sy, c[i]);          // links oben

    for (i = 0, sx = GrSizeX(), sy = 0 ; i < 16; i++, sx -= dx)
        GrLine (mx, my, sx, sy, c[i]);          // rechts oben

    for (i = 0, sx = GrSizeX(), sy = 0 ; i < 16; i++, sy += dy)
        GrLine (mx, my, sx, sy, c[i]);          // rechts oben

    for (i = 0, sx = GrSizeX(), sy = GrSizeY() ; i < 16; i++, sy -= dy)
        GrLine (mx, my, sx, sy, c[i]);          // rechts unten

    for (i = 0, sx = GrSizeX(), sy = GrSizeY() ; i < 16; i++, sx -= dx)
        GrLine (mx, my, sx, sy, c[i]);          // rechts unten

    for (i = 0, sx = 0, sy = GrSizeY() ; i < 16; i++, sx += dx)
        GrLine (mx, my, sx, sy, c[i]);          // links unten

    for (i = 0, sx = 0, sy = GrSizeY() ; i < 16; i++, sy -= dy)
        GrLine (mx, my, sx, sy, c[i]);          // links unten

    for (i = 0, sx = 0, sy = 0 ; i < 16; i++, sy += dy)
        GrLine (mx, my, sx, sy, c[i]);          // links oben
}


void Palette_256 (void)
{
    // Ausgabe in 256 Farben

    for (int i = 0; i < 256; i++)
    {
        // GRAllocColor bentigt R/G/B-Werte zwischen 0 und 255

        // Die niedrigsten 2 Bits geben die Hintergrund-Intensitt an 
        // (63, 127, 191, 255)

        int y = ((i & 3) + 1) * 256 / 4 - 1;

        int b = (((i >> 2) & 3) + 1) * y / 4;           // blau
        int g = (((i >> 4) & 3) + 1) * y / 4;           // grn
        int r = (((i >> 6) & 3) + 1) * y / 4;           // rot

        c[i] = GrAllocColor (r, g, b);
    }
}


void Palette_256_Reinigen (void)
{
    for (int i = 0; i < 256; i++)
        GrFreeColor (c[i]);
}


void Testbild_256 (void)
{
    char buf[80];

    int dx = GrSizeX() / 8;
    int dy = GrSizeY() / 32;

    int x, y;
    int i, j;

    int r, g, b, col;

    for (i = 0, y = dy/2; i < 32; i++, y += dy)         // Zeilen
    {
        for (j = 0, x = 0; j < 8; j++, x += dx)         // Spalten
        {
            GrQueryColor (col = c[i*8 + j], &r, &g, &b);

            sprintf (buf, " %02x%02x%02x ", r, g, b);

            GrTextXY (x, y, buf, GrWhite(), col);
        }
    }
}


